@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: gtkimcontextscim.cpp,v 1.168 2005/07/03 08:36:41 suzhe Exp $
+ * $Id: gtkimcontextscim.cpp,v 1.170.2.1 2005/12/30 08:50:09 suzhe Exp $
  */
 
 #define Uses_SCIM_DEBUG
@@ -1039,7 +1039,9 @@ static void
 panel_slot_process_helper_event (int context, const String &target_uuid, const String &helper_uuid, const Transaction &trans)
 {
     GtkIMContextSCIM *ic = find_ic (context);
-    SCIM_DEBUG_FRONTEND(1) << "panel_slot_process_helper_event context=" << context << " target=" << target_uuid << " helper=" << helper_uuid << " ic=" << ic << "\n";
+    SCIM_DEBUG_FRONTEND(1) << "panel_slot_process_helper_event context=" << context << " target=" << target_uuid
+                           << " helper=" << helper_uuid << " ic=" << ic << " ic->impl=" << (ic ? ic->impl : 0) << " ic-uuid="
+                           << ((ic && ic->impl) ? ic->impl->si->get_factory_uuid () : "" ) << "\n";
     if (ic && ic->impl && ic->impl->si->get_factory_uuid () == target_uuid) {
         _panel_client.prepare (ic->id);
         SCIM_DEBUG_FRONTEND(2) << "call process_helper_event\n";
@@ -1287,11 +1289,14 @@ panel_initialize ()
 {
     String display_name;
 
+    {
 #if GDK_MULTIHEAD_SAFE
-    display_name = gdk_display_get_name (gdk_display_get_default ());
+        const char *p = gdk_display_get_name (gdk_display_get_default ());
 #else
-    display_name = getenv ("DISPLAY");
+        const char *p = getenv ("DISPLAY");
 #endif
+        if (p) display_name = String (p);
+    }
 
     SCIM_DEBUG_FRONTEND(1) << "panel_initialize..\n";
 
@@ -1351,6 +1356,7 @@ turn_on_ic (GtkIMContextSCIM *ic)
         ic->impl->is_on = true;
 
         if (ic == _focused_ic) {
+            panel_req_focus_in (ic);
             panel_req_update_screen (ic);
             panel_req_update_spot_location (ic);
             panel_req_update_factory_info (ic);
@@ -1690,9 +1696,12 @@ initialize (void)
 
     if (_backend.null ()) {
         fprintf (stderr, "GTK IM Module SCIM: Cannot create BackEnd Object!\n");
+    } else {
+        _fallback_factory = _backend->get_factory (SCIM_COMPOSE_KEY_FACTORY_UUID);
     }
 
-    _fallback_factory = new ComposeKeyFactory ();
+    if (_fallback_factory.null ()) _fallback_factory = new DummyIMEngineFactory ();
+
     _fallback_instance = _fallback_factory->create_instance (String ("UTF-8"), 0);
     _fallback_instance->signal_connect_commit_string (slot (fallback_commit_string_cb));
 
@@ -2130,7 +2139,9 @@ slot_start_helper (IMEngineInstanceBase *si,
 {
     GtkIMContextSCIM *ic = static_cast<GtkIMContextSCIM *> (si->get_frontend_data ());
 
-    SCIM_DEBUG_FRONTEND(1) << "slot_start_helper helper= " << helper_uuid << " context=" << (ic ? ic->id : -1) << " ic=" << ic << "...\n";
+    SCIM_DEBUG_FRONTEND(1) << "slot_start_helper helper= " << helper_uuid << " context="
+                           << (ic ? ic->id : -1) << " ic=" << ic
+                           << " ic-uuid=" << ((ic && ic->impl) ? ic->impl->si->get_factory_uuid () : "") << "...\n";
 
     if (ic && ic->impl)
         _panel_client.start_helper (ic->id, helper_uuid);
@@ -2155,7 +2166,9 @@ slot_send_helper_event (IMEngineInstanceBase *si,
 {
     GtkIMContextSCIM *ic = static_cast<GtkIMContextSCIM *> (si->get_frontend_data ());
 
-    SCIM_DEBUG_FRONTEND(1) << "slot_send_helper_event helper= " << helper_uuid << " context=" << (ic ? ic->id : -1) << " ic=" << ic << "...\n";
+    SCIM_DEBUG_FRONTEND(1) << "slot_send_helper_event helper= " << helper_uuid << " context="
+                           << (ic ? ic->id : -1) << " ic=" << ic
+                           << " ic-uuid=" << ((ic && ic->impl) ? ic->impl->si->get_factory_uuid () : "") << "...\n";
 
     if (ic && ic->impl)
         _panel_client.send_helper_event (ic->id, helper_uuid, trans);
