@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_x11_frontend.cpp,v 1.177 2005/06/30 10:25:14 suzhe Exp $
+ * $Id: scim_x11_frontend.cpp,v 1.179 2005/08/16 07:26:54 suzhe Exp $
  *
  */
 
@@ -413,7 +413,12 @@ X11FrontEnd::init (int argc, char **argv)
     if (m_panel_client.open_connection (m_config->get_name (), m_display_name) < 0)
         throw FrontEndError (String ("X11 -- failed to connect to the panel daemon!"));
 
-    m_fallback_factory = new ComposeKeyFactory ();
+    // Only use ComposeKeyFactory when it's enabled.
+    if (validate_factory (SCIM_COMPOSE_KEY_FACTORY_UUID, "UTF-8"))
+        m_fallback_factory = new ComposeKeyFactory ();
+    else
+        m_fallback_factory = new DummyIMEngineFactory ();
+
     m_fallback_instance = m_fallback_factory->create_instance (String ("UTF-8"), 0);
     m_fallback_instance->signal_connect_commit_string (slot (this, &X11FrontEnd::fallback_commit_string_cb));
 }
@@ -1464,8 +1469,10 @@ X11FrontEnd::ims_turn_on_ic (X11IC *ic)
         if (m_shared_input_method)
             m_config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), true);
 
-        if (is_focused_ic (ic))
+        if (is_focused_ic (ic)) {
+            panel_req_focus_in (ic);
             start_ic (ic);
+        }
     }
 }
 
