@@ -5,7 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -23,11 +23,10 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_x11_ic.cpp,v 1.14 2004/08/14 03:26:38 suzhe Exp $
+ * $Id: scim_x11_ic.cpp,v 1.19 2005/06/26 16:35:12 suzhe Exp $
  *
  */
 
-#define Uses_STL_MAP
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
@@ -74,7 +73,7 @@ X11ICManager::~X11ICManager ()
 X11IC *
 X11ICManager::new_ic ()
 {
-    static CARD16 icid = 0;
+    static CARD16 base_icid = 1;
     X11IC *rec;
     
     if (m_free_list != NULL) {
@@ -84,7 +83,9 @@ X11ICManager::new_ic ()
         rec = new X11IC;
     }
 
-    rec->id = ++ icid;
+    if (base_icid == 0) base_icid = 1;
+
+    rec->icid = base_icid ++;
 
     rec->next = m_ic_list;
     m_ic_list = rec;
@@ -97,7 +98,7 @@ X11ICManager::delete_ic (CARD16 icid)
     X11IC *rec, *last = NULL;
 
     for (rec = m_ic_list; rec != NULL; last = rec, rec = rec->next) {
-        if (rec->id == icid) {
+        if (rec->icid == icid) {
             if (last != NULL)
                 last->next = rec->next;
             else
@@ -106,11 +107,12 @@ X11ICManager::delete_ic (CARD16 icid)
             rec->next = m_free_list;
             m_free_list = rec;
 
-            rec->si_id = -1;
-            rec->id = 0;
+            rec->siid = -1;
+            rec->icid = 0;
             rec->connect_id = 0;
             rec->client_win = 0;
             rec->focus_win = 0;
+            rec->shared_siid = false;
             rec->xims_on = false;
             rec->encoding = String ();
             rec->locale = String ();
@@ -133,12 +135,6 @@ X11ICManager::get_connection_locale (CARD16 connect_id)
     return String ();
 }
 
-String
-X11ICManager::get_connection_encoding (CARD16 connect_id)
-{
-    return scim_get_locale_encoding (get_connection_locale (connect_id));
-}
-
 void
 X11ICManager::new_connection (IMOpenStruct *call_data)
 {
@@ -169,9 +165,10 @@ X11ICManager::create_ic (IMChangeICStruct *call_data, int siid)
     rec = new_ic ();
     if (rec == NULL) return 0;
 
-    call_data->icid = rec->id;
+    call_data->icid = rec->icid;
     rec->connect_id = call_data->connect_id;
-    rec->si_id = siid;
+    rec->siid = siid;
+    rec->shared_siid = false;
     rec->xims_on = false;
     rec->onspot_preedit_started = false;
     rec->onspot_preedit_length = 0;
@@ -190,7 +187,7 @@ X11ICManager::find_ic (CARD16 icid)
 {
     X11IC *rec = m_ic_list;
     while (rec != NULL) {
-        if (rec->id == icid)
+        if (rec->icid == icid)
             return rec;
         rec = rec->next;
     }
@@ -202,7 +199,7 @@ X11ICManager::find_ic_by_siid (int siid)
 {
     X11IC *rec = m_ic_list;
     while (rec != NULL) {
-        if (rec->si_id == siid)
+        if (rec->siid == siid)
             return rec;
         rec = rec->next;
     }
