@@ -23,12 +23,13 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_rawcode_imengine.cpp,v 1.6 2005/03/16 15:33:06 suzhe Exp $
+ * $Id: scim_rawcode_imengine.cpp,v 1.7.2.5 2006/01/12 07:00:36 suzhe Exp $
  *
  */
 
 #define Uses_SCIM_IMENGINE
 #define Uses_SCIM_ICONV
+#define Uses_C_STRING
 #define Uses_SCIM_CONFIG_BASE
 #define Uses_SCIM_CONFIG_PATH
 #include "scim_private.h"
@@ -40,19 +41,18 @@
 #define scim_imengine_module_init rawcode_LTX_scim_imengine_module_init
 #define scim_imengine_module_create_factory rawcode_LTX_scim_imengine_module_create_factory
 
-#define SCIM_CONFIG_IMENGINE_RAWCODE_LANGUAGES "/IMEngine/RawCode/Languages"
-
-#define SCIM_PROP_STATUS                       "/IMEngine/RawCode/Status"
+#define SCIM_CONFIG_IMENGINE_RAWCODE_LOCALES   "/IMEngine/RawCode/Locales"
+#define SCIM_PROP_RAWCODE_ENCODING             "/IMEngine/RawCode/Encoding"
 
 #define SCIM_RAWCODE_ICON_FILE                 (SCIM_ICONDIR "/rawcode.png")
 
 using namespace scim;
 
-static Pointer <RawCodeFactory> _scim_rawcode_factory;
-static ConfigPointer _scim_config;
+static Pointer <RawCodeFactory> __rawcode_factory;
 
-static const char * _DEFAULT_LANGUAGES = N_(
-    "zh_CN,zh_TW,zh_HK,zh_SG,ja_JP,ko_KR");
+static String __rawcode_locales ("zh_CN.GB18030,zh_CN.GBK,zh_CN.GB2312,zh_TW,zh_TW.EUC-TW,zh_HK,ja_JP,ja_JP.sjis,ko_KR,en_US.UTF-8");
+
+static std::vector<String> __rawcode_encodings;
 
 extern "C" {
     void scim_module_init (void)
@@ -61,13 +61,30 @@ extern "C" {
 
     void scim_module_exit (void)
     {
-        _scim_rawcode_factory.reset ();
-        _scim_config.reset ();
+        __rawcode_factory.reset ();
     }
 
     unsigned int scim_imengine_module_init (const ConfigPointer &config)
     {
-        _scim_config = config;
+        if (!config.null ()) {
+            String locales = config->read (String (SCIM_CONFIG_IMENGINE_RAWCODE_LOCALES), String ("default"));
+            if (locales != "default")
+                __rawcode_locales = locales;
+        }
+
+        std::vector <String> locale_list;
+
+        scim_split_string_list (locale_list, __rawcode_locales);
+
+        for (size_t i=0; i<locale_list.size (); ++i) {
+            locale_list [i] = scim_validate_locale (locale_list [i]);
+            if (locale_list [i].length ())
+                __rawcode_encodings.push_back (scim_get_locale_encoding (locale_list [i]));
+        }
+
+        std::sort (__rawcode_encodings.begin (), __rawcode_encodings.end ());
+        __rawcode_encodings.erase (std::unique (__rawcode_encodings.begin (), __rawcode_encodings.end ()), __rawcode_encodings.end ());
+
         return 1;
     }
 
@@ -77,42 +94,17 @@ extern "C" {
 
         if (factory != 0) return NULL;
 
-        if (!_scim_config.null ()) {
-            languages = _scim_config->read (
-                        String (SCIM_CONFIG_IMENGINE_RAWCODE_LANGUAGES),
-                        String ("default"));
-        } else {
-            languages = String ("default");
-        }
+        if (__rawcode_factory.null ())
+            __rawcode_factory = new RawCodeFactory ();
 
-        if (_scim_rawcode_factory.null ()) {
-            _scim_rawcode_factory =
-                new RawCodeFactory (utf8_mbstowcs (String (_("RAW CODE"))), languages);
-        }
-        return _scim_rawcode_factory;
+        return __rawcode_factory;
     }
 }
 
 // implementation of RawCode
 RawCodeFactory::RawCodeFactory ()
 {
-    m_name = utf8_mbstowcs (_("RAW CODE"));
-    set_languages (String (_(_DEFAULT_LANGUAGES)));
-}
-
-RawCodeFactory::RawCodeFactory (const WideString& name,
-                                const String& languages)
-{
-    // do not allow too long name
-    if (name.length () <= 8)
-        m_name = name;
-    else
-        m_name.assign (name.begin (), name.begin () + 8);
-
-    if (languages == String ("default"))
-        set_languages (String (_(_DEFAULT_LANGUAGES)));
-    else
-        set_languages (languages);
+    set_locales (__rawcode_locales);
 }
 
 RawCodeFactory::~RawCodeFactory ()
@@ -122,14 +114,14 @@ RawCodeFactory::~RawCodeFactory ()
 WideString
 RawCodeFactory::get_name () const
 {
-    return m_name;
+    return utf8_mbstowcs (_("RAW CODE"));
 }
 
 WideString
 RawCodeFactory::get_authors () const
 {
     return utf8_mbstowcs (String (
-                _("(C) 2002-2005 James Su <suzhe@tsinghua.org.cn>")));
+                _("(C) 2002-2006 James Su <suzhe@tsinghua.org.cn>")));
 }
 
 WideString
@@ -145,10 +137,6 @@ RawCodeFactory::get_help () const
                             "Hot Keys:\n\n"
                             "  Control+u:\n"
                             "    switch between Multibyte encoding and Unicode.\n\n"
-                            "  Control+comma:\n"
-                            "    switch between full/half width punctuation mode.\n\n"
-                            "  Shift+space:\n"
-                            "    switch between full/half width letter mode.\n\n"
                             "  Esc:\n"
                             "    reset the input method.\n")));
 }
@@ -180,29 +168,31 @@ RawCodeFactory::create_instance (const String& encoding, int id)
 int
 RawCodeFactory::get_maxlen (const String &encoding)
 {
+    if (encoding == "UTF-8") return 4;
+    if (encoding == "Unicode") return 0;
+
     std::vector <String> locales;
 
     scim_split_string_list (locales, get_locales ());
 
-    for (unsigned int i=0; i<locales.size (); ++i)
+    for (size_t i=0; i<locales.size (); ++i)
         if (scim_get_locale_encoding (locales [i]) == encoding)
             return scim_get_locale_maxlen (locales [i]);
-    return 1;
+
+    return 0;
 }
 
 // implementation of RawCodeInstance
 RawCodeInstance::RawCodeInstance (RawCodeFactory *factory,
-                                              const String& encoding,
-                                              int id)
+                                  const String& encoding,
+                                  int id)
     : IMEngineInstanceBase (factory, encoding, id),
-      m_factory (factory),
-      m_status_property (SCIM_PROP_STATUS, _("Unicode"), "", _("The status of the current input method. Click to change it.")),
-      m_unicode (true),
-      m_forward (false),
-      m_focused (false),
-      m_max_preedit_len (4),
-      m_iconv (encoding)
+      m_factory (factory)
 {
+    if (!m_client_iconv.set_encoding (encoding))
+        m_client_iconv.set_encoding ("UTF-8");
+
+    set_working_encoding ("Unicode");
 }
 
 RawCodeInstance::~RawCodeInstance ()
@@ -212,87 +202,75 @@ RawCodeInstance::~RawCodeInstance ()
 bool
 RawCodeInstance::process_key_event (const KeyEvent& key)
 {
-    if (!m_focused) return false;
+    if (key.is_key_release ()) return true;
+
+    // switch between unicode and native code mode
+    if ((key.code == SCIM_KEY_u || key.code == SCIM_KEY_U) && key.is_control_down ()) {
+        if (m_unicode)
+            set_working_encoding (get_encoding ());
+        else
+            set_working_encoding ("Unicode");
 
-    // capture the state switch key events
-    if ( ((key.code == SCIM_KEY_Alt_L || key.code == SCIM_KEY_Alt_R) &&
-            key.is_shift_down ()) ||
-         ((key.code == SCIM_KEY_Shift_L || key.code == SCIM_KEY_Shift_R) && 
-            (key.is_alt_down () || key.is_control_down ())) &&
-         key.is_key_press ()) {
-        m_forward = !m_forward;
-        refresh_status_property ();
         reset ();
         return true;
     }
 
-    if (key.is_key_release ()) return true;
+    //reset key
+    if (key.code == SCIM_KEY_Escape && key.mask == 0) {
+        reset ();
+        return true;
+    }
 
-    if (!m_forward) {
-        // switch between unicode and native code mode
-        if ((key.code == SCIM_KEY_u || key.code == SCIM_KEY_U) &&
-             key.is_control_down ()) {
-            m_unicode = !m_unicode;
-            refresh_status_property ();
+    //delete key
+    if (key.code == SCIM_KEY_BackSpace && key.mask == 0 &&
+        m_preedit_string.size () != 0) {
+        m_preedit_string.erase (m_preedit_string.length () - 1, 1);
+        update_preedit_string (m_preedit_string);
+        update_preedit_caret (m_preedit_string.length ());
+        process_preedit_string ();
+        return true;
+    }
+
+    // valid keys
+    if (((key.code >= SCIM_KEY_0 && key.code <= SCIM_KEY_9) ||
+         (key.code >= SCIM_KEY_A && key.code <= SCIM_KEY_F) ||
+         (key.code >= SCIM_KEY_a && key.code <= SCIM_KEY_f)) &&
+        (key.mask == 0 || key.is_shift_down ()) &&
+        m_preedit_string.length () < m_max_preedit_len) {
+
+        if (m_preedit_string.length () == 0)
+            show_preedit_string ();
+
+        ucs4_t ascii = (ucs4_t) tolower (key.get_ascii_code ());
+        m_preedit_string.push_back (ascii);
+        update_preedit_string (m_preedit_string);
+        update_preedit_caret (m_preedit_string.length ());
+        process_preedit_string ();
+        return true;
+    }
+
+    // commit key
+    if (key.code == SCIM_KEY_space && key.mask == 0 && m_preedit_string.length ()
+        && m_lookup_table.number_of_candidates ()) {
+        WideString str = m_lookup_table.get_candidate_label (0);
+        // It's already a complete char, commit it.
+        if (str.length () && str [0] == 0x20) {
+            commit_string (m_lookup_table.get_candidate_in_current_page (0));
             reset ();
             return true;
         }
-
-        //reset key
-        if (key.code == SCIM_KEY_Escape && key.mask == 0) {
-            reset ();
-            return true;
-        }
-
-        //delete key
-        if (key.code == SCIM_KEY_BackSpace && key.mask == 0 &&
-            m_preedit_string.size () != 0) {
-            m_preedit_string.erase (m_preedit_string.length () - 1, 1);
-            update_preedit_string (m_preedit_string);
-            update_preedit_caret (m_preedit_string.length ());
-            process_preedit_string ();
-            return true;
-        }
-
-        // valid keys
-        if (((key.code >= SCIM_KEY_0 && key.code <= SCIM_KEY_9) ||
-             (key.code >= SCIM_KEY_A && key.code <= SCIM_KEY_F) ||
-             (key.code >= SCIM_KEY_a && key.code <= SCIM_KEY_f)) &&
-            (key.mask == 0 || key.is_shift_down ()) &&
-            m_preedit_string.length () < m_max_preedit_len) {
-
-            if (m_preedit_string.length () == 0)
-                show_preedit_string ();
-
-            ucs4_t ascii = (ucs4_t) tolower (key.get_ascii_code ());
-            m_preedit_string.push_back (ascii);
-            update_preedit_string (m_preedit_string);
-            update_preedit_caret (m_preedit_string.length ());
-            process_preedit_string ();
-            return true;
-        }
-
-        //page up key.
-        if (key.code == SCIM_KEY_comma && key.mask == 0 &&
-            m_preedit_string.length () && m_lookup_table.number_of_candidates ()) {
-            m_lookup_table.page_up ();
-            if (create_lookup_table (m_lookup_table.get_current_page_start ()) > 0)
-                update_lookup_table (m_lookup_table);
-            else
-                hide_lookup_table ();
-        }
-
-        //page down key.
-        if (key.code == SCIM_KEY_period && key.mask == 0 &&
-            m_preedit_string.length () && m_lookup_table.number_of_candidates ()) {
-            m_lookup_table.page_down ();
-            if (create_lookup_table (m_lookup_table.get_current_page_start ()) > 0)
-                update_lookup_table (m_lookup_table);
-            else
-                hide_lookup_table ();
-        }
     }
 
+    //page up key.
+    if ((key.code == SCIM_KEY_comma || key.code == SCIM_KEY_minus || key.code == SCIM_KEY_bracketleft
+         || key.code == SCIM_KEY_Page_Up) && key.mask == 0)
+        lookup_table_page_up ();
+
+    //page down key.
+    if ((key.code == SCIM_KEY_period || key.code == SCIM_KEY_equal || key.code == SCIM_KEY_bracketright
+         || key.code == SCIM_KEY_Page_Down) && key.mask == 0)
+        lookup_table_page_down ();
+
     //other keys is not allowed when preediting
     if (m_preedit_string.length ())
         return true;
@@ -320,12 +298,14 @@ RawCodeInstance::lookup_table_page_up ()
 {
     if (m_preedit_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.page_up ();
-        if (create_lookup_table (m_lookup_table.get_current_page_start ()) > 0) {
-            update_lookup_table (m_lookup_table);
-            show_lookup_table ();
-        } else {
-            hide_lookup_table ();
-        }
+        m_lookup_table.set_page_size (m_lookup_table.number_of_candidates ());
+
+        m_lookup_table.set_candidate_labels (
+            std::vector <WideString> (
+                m_lookup_table_labels.begin () + m_lookup_table.get_current_page_start (),
+                m_lookup_table_labels.end ()));
+
+        update_lookup_table (m_lookup_table);
     }
 }
 
@@ -334,12 +314,14 @@ RawCodeInstance::lookup_table_page_down ()
 {
     if (m_preedit_string.length () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.page_down ();
-        if (create_lookup_table (m_lookup_table.get_current_page_start ()) > 0) {
-            update_lookup_table (m_lookup_table);
-            show_lookup_table ();
-        } else {
-            hide_lookup_table ();
-        }
+        m_lookup_table.set_page_size (m_lookup_table.number_of_candidates ());
+
+        m_lookup_table.set_candidate_labels (
+            std::vector <WideString> (
+                m_lookup_table_labels.begin () + m_lookup_table.get_current_page_start (),
+                m_lookup_table_labels.end ()));
+
+        update_lookup_table (m_lookup_table);
     }
 }
 
@@ -351,24 +333,38 @@ RawCodeInstance::move_preedit_caret (unsigned int /*pos*/)
 void
 RawCodeInstance::reset ()
 {
+    if (!m_client_iconv.set_encoding (get_encoding ()))
+        m_client_iconv.set_encoding ("UTF-8");
+
     m_preedit_string = WideString ();
 
-    if (m_unicode) m_max_preedit_len = 4;
-    else if (m_factory)
-        m_max_preedit_len = m_factory->get_maxlen (get_encoding ()) * 2;
-
-    m_iconv.set_encoding (get_encoding ());
     m_lookup_table.clear ();
 
     hide_lookup_table ();
     hide_preedit_string ();
 }
 
+void
+RawCodeInstance::set_working_encoding (const String &encoding)
+{
+    unsigned int maxlen = m_factory->get_maxlen (encoding);
+
+    if (maxlen && encoding != "Unicode" && m_working_iconv.set_encoding (encoding)){
+        m_unicode = false;
+        m_max_preedit_len = maxlen * 2;
+        m_working_encoding = encoding;
+    } else {
+        m_unicode = true;
+        m_working_encoding = "Unicode";
+        m_max_preedit_len = 6;
+    }
+
+    refresh_encoding_property ();
+}
+
 void
 RawCodeInstance::focus_in ()
 {
-    m_focused = true;
-
     initialize_properties ();
 
     if (m_preedit_string.length ()) {
@@ -385,21 +381,15 @@ RawCodeInstance::focus_in ()
 void
 RawCodeInstance::focus_out ()
 {
-    m_focused = false;
+    reset ();
 }
 
 void
 RawCodeInstance::trigger_property (const String &property)
 {
-    if (property == SCIM_PROP_STATUS) {
-        if (m_forward)
-            m_forward = false;
-        else if (m_unicode)
-            m_unicode = false;
-        else
-            m_unicode = m_forward = true;
-
-        refresh_status_property ();
+    if (property.substr (0, strlen (SCIM_PROP_RAWCODE_ENCODING)) == SCIM_PROP_RAWCODE_ENCODING) {
+        set_working_encoding (property.substr (strlen (SCIM_PROP_RAWCODE_ENCODING) + 1));
+        reset ();
     }
 }
 
@@ -408,24 +398,26 @@ RawCodeInstance::initialize_properties ()
 {
     PropertyList proplist;
 
-    proplist.push_back (m_status_property);
+    proplist.push_back (Property (SCIM_PROP_RAWCODE_ENCODING,
+                                  _(m_working_encoding.c_str ()),
+                                  "",
+                                  _("The status of the current input method. Click to change it.")));
+
+    proplist.push_back (Property (String (SCIM_PROP_RAWCODE_ENCODING) + String ("/Unicode"), _("Unicode")));
+
+    for (size_t i = 0; i < __rawcode_encodings.size (); ++i)
+        proplist.push_back (Property (String (SCIM_PROP_RAWCODE_ENCODING) + String ("/") + __rawcode_encodings [i], _(__rawcode_encodings [i].c_str ())));
 
     register_properties (proplist);
 }
 
 void
-RawCodeInstance::refresh_status_property ()
+RawCodeInstance::refresh_encoding_property ()
 {
-    if (m_focused) {
-        if (m_forward)
-            m_status_property.set_label (_("En"));
-        else if (m_unicode)
-            m_status_property.set_label (_("Unicode"));
-        else
-            m_status_property.set_label (get_encoding ());
-
-        update_property (m_status_property);
-    }
+    update_property (Property (SCIM_PROP_RAWCODE_ENCODING,
+                               _(m_working_encoding.c_str ()),
+                               "",
+                               _("The status of the current input method. Click to change it.")));
 }
 
 void
@@ -438,21 +430,20 @@ RawCodeInstance::process_preedit_string ()
     }
 
     if (m_unicode) {
-        if (m_preedit_string.length () == 3 &&
-            create_lookup_table () > 0) {
+        if (m_preedit_string.length () >= 3 && m_preedit_string.length () < 6 && create_lookup_table () > 0) {
             update_lookup_table (m_lookup_table);
-        } else if (m_preedit_string.length () == 4) {
+        } else if (m_preedit_string.length () == 6) {
             WideString str;
             ucs4_t code = get_unicode_value (m_preedit_string);
 
+            m_preedit_string = WideString ();
+            m_lookup_table.clear ();
+            hide_preedit_string ();
+
             // If code is valid under current encoding,
             // then commit it.
-            if (m_iconv.test_convert (&code, 1)) {
+            if (m_client_iconv.test_convert (&code, 1) && code > 0 && code < 0x10FFFF) {
                 str.push_back (code);
-                m_preedit_string = WideString ();
-                m_lookup_table.clear ();
-
-                hide_preedit_string ();
                 commit_string (str);
             }
         } else if (m_lookup_table.number_of_candidates ()){
@@ -460,15 +451,15 @@ RawCodeInstance::process_preedit_string ()
         }
     } else {
         String code = get_multibyte_string (m_preedit_string);
-        WideString str;
+        WideString wstr;
 
         // convert ok, then commit.
-        if (m_iconv.convert (str, code) && str.length () > 0 && str [0] >= 128) {
+        if (m_working_iconv.convert (wstr, code) && wstr.length () > 0 && wstr [0] >= 128 && m_client_iconv.test_convert (wstr)) {
             m_preedit_string = WideString ();
             m_lookup_table.clear ();
 
             hide_preedit_string ();
-            commit_string (str);
+            commit_string (wstr);
         } else if (create_lookup_table () > 0) {
             update_lookup_table (m_lookup_table);
         }
@@ -509,7 +500,7 @@ RawCodeInstance::get_multibyte_string (const WideString & preedit)
     if (preedit.length () == 0)
         return str;
 
-    for (unsigned int i=0; i<preedit.length (); ++i) {
+    for (size_t i=0; i<preedit.length (); ++i) {
         if (i % 2 == 0)
             ch = (char) ascii_to_hex ((int) preedit [i]) & 0x0f;
         else {
@@ -529,48 +520,55 @@ ucs4_t
 RawCodeInstance::get_unicode_value (const WideString &preedit)
 {
     ucs4_t code = 0;
-    for (unsigned int i=0; i<preedit.length (); ++i) {
+    for (size_t i=0; i<preedit.length (); ++i) {
         code = (code << 4) | (ascii_to_hex ((int) preedit [i]) & 0x0f);
     }
     return code;
 }
 
 int
-RawCodeInstance::create_lookup_table (int start)
+RawCodeInstance::create_lookup_table ()
 {
-    std::vector<WideString> labels;
     String mbs_code;
     ucs4_t ucs_code;
     WideString trail;
-    WideString str;
+    WideString wstr;
 
     m_lookup_table.clear ();
+    m_lookup_table_labels.clear ();
 
-    trail.push_back (0);
+    trail.push_back (0x20);
 
-    for (int i=start; i<16; ++i) {
+    if (m_unicode) {
+        ucs_code = get_unicode_value (m_preedit_string);
+        if (m_client_iconv.test_convert (&ucs_code, 1) && ucs_code > 0 && ucs_code < 0x10FFFF) {
+            m_lookup_table_labels.push_back (trail);
+            m_lookup_table.append_candidate (ucs_code);
+        }
+    }
+
+    for (int i=0; i<16; ++i) {
         trail [0] = (ucs4_t) hex_to_ascii (i);
         if (m_unicode) {
             ucs_code = get_unicode_value (m_preedit_string + trail);
-            if (m_iconv.test_convert (&ucs_code, 1)) {
-                labels.push_back (trail);
+            if (m_client_iconv.test_convert (&ucs_code, 1) && ucs_code > 0 && ucs_code < 0x10FFFF) {
+                m_lookup_table_labels.push_back (trail);
                 m_lookup_table.append_candidate (ucs_code);
             }
         } else {
             mbs_code = get_multibyte_string (m_preedit_string + trail);
-            if (m_iconv.convert (str, mbs_code) && str.length () > 0 && str [0] >= 128) {
-                labels.push_back (trail);
-                m_lookup_table.append_candidate (str [0]);
+            if (m_working_iconv.convert (wstr, mbs_code) && wstr.length () > 0 && wstr [0] >= 128 && m_client_iconv.test_convert (wstr)) {
+                m_lookup_table_labels.push_back (trail);
+                m_lookup_table.append_candidate (wstr);
             }
         }
     }
 
-    m_lookup_table.set_page_size (labels.size ());
-    m_lookup_table.set_candidate_labels (labels);
+    m_lookup_table.set_page_size (m_lookup_table_labels.size ());
+    m_lookup_table.set_candidate_labels (m_lookup_table_labels);
 
-    return labels.size ();
+    return m_lookup_table_labels.size ();
 }
-
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
