@@ -5,7 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket_imengine.cpp,v 1.6 2004/07/18 13:14:36 suzhe Exp $
+ * $Id: scim_socket_imengine.cpp,v 1.21 2005/07/06 03:57:04 suzhe Exp $
  *
  */
 
@@ -31,14 +31,14 @@
 #define Uses_SCIM_CONFIG_BASE
 #define Uses_SCIM_CONFIG_PATH
 #define Uses_SCIM_SOCKET
-#define Uses_SCIM_SOCKET_TRANSACTION
+#define Uses_SCIM_TRANSACTION
 #define Uses_C_STDLIB
-#define Uses_STL_MAP
 
+#include <unistd.h>
 #include "scim_private.h"
 #include "scim.h"
 #include "scim_socket_imengine.h"
-#include <unistd.h>
+#include "scim_stl_map.h"
 
 #define scim_module_init                    socket_LTX_scim_module_init
 #define scim_module_exit                    socket_LTX_scim_module_exit
@@ -56,8 +56,16 @@
 
 using namespace scim;
 
-class SocketIMEngineGlobal
+class scim::SocketIMEngineGlobal
 {
+#if SCIM_USE_STL_EXT_HASH_MAP
+typedef __gnu_cxx::hash_map <String, String, scim_hash_string> IconRepository;
+#elif SCIM_USE_STL_HASH_MAP
+typedef std::hash_map <String, String, scim_hash_string> IconRepository;
+#else
+typedef std::map <String, String> IconRepository;
+#endif
+
     SocketClient             m_socket_client;
     SocketAddress            m_socket_address;
     uint32                   m_socket_magic_key;
@@ -65,7 +73,7 @@ class SocketIMEngineGlobal
 
     std::vector<String>      m_peer_factories;
 
-    std::map<String, String> m_icon_repository;
+    IconRepository           m_icon_repository;
 
     Signal0<void>            m_signal_reconnect;
 
@@ -77,9 +85,9 @@ public:
     unsigned int    number_of_factories ();
     SocketFactory * create_factory (unsigned int index);
 
-    void            init_transaction (SocketTransaction &trans);
-    bool            send_transaction (SocketTransaction &trans);
-    bool            receive_transaction (SocketTransaction &trans);
+    void            init_transaction (Transaction &trans);
+    bool            send_transaction (Transaction &trans);
+    bool            receive_transaction (Transaction &trans);
 
     String          load_icon (const String &icon);
 
@@ -132,6 +140,8 @@ extern "C" {
     }
 }
 
+namespace scim {
+
 SocketIMEngineGlobal::SocketIMEngineGlobal ()
     : m_socket_magic_key (0),
       m_socket_timeout (-1)
@@ -167,7 +177,7 @@ SocketIMEngineGlobal::init ()
                          << ") MagicKey (" << m_socket_magic_key << ").\n";
 
     // Init the connection, and get IMEngineFactory list.
-    SocketTransaction trans;
+    Transaction trans;
 
     // Get IMEngineFactory list.
     init_transaction (trans);
@@ -195,11 +205,11 @@ SocketIMEngineGlobal::create_connection ()
     if (!m_socket_client.connect (m_socket_address))
         return false;
 
-    if (!scim_socket_trans_open_connection (m_socket_magic_key,
-                                            String ("SocketIMEngine"),
-                                            String ("SocketFrontEnd"),
-                                            m_socket_client,
-                                            m_socket_timeout)) {
+    if (!scim_socket_open_connection (m_socket_magic_key,
+                                      String ("SocketIMEngine"),
+                                      String ("SocketFrontEnd"),
+                                      m_socket_client,
+                                      m_socket_timeout)) {
         m_socket_client.close ();
         return false;
     }
@@ -235,7 +245,7 @@ SocketIMEngineGlobal::create_factory (unsigned int index)
 }
 
 void
-SocketIMEngineGlobal::init_transaction (SocketTransaction &trans)
+SocketIMEngineGlobal::init_transaction (Transaction &trans)
 {
     trans.clear ();
     trans.put_command (SCIM_TRANS_CMD_REQUEST);
@@ -243,13 +253,13 @@ SocketIMEngineGlobal::init_transaction (SocketTransaction &trans)
 }
 
 bool
-SocketIMEngineGlobal::send_transaction (SocketTransaction &trans)
+SocketIMEngineGlobal::send_transaction (Transaction &trans)
 {
     return trans.write_to_socket (m_socket_client);
 }
 
 bool
-SocketIMEngineGlobal::receive_transaction (SocketTransaction &trans)
+SocketIMEngineGlobal::receive_transaction (Transaction &trans)
 {
     return trans.read_from_socket (m_socket_client, m_socket_timeout);
 }
@@ -259,7 +269,7 @@ SocketIMEngineGlobal::load_icon (const String &icon)
 {
     String local_icon = icon;
 
-    std::map <String, String>::const_iterator it = m_icon_repository.find (icon);
+    IconRepository::const_iterator it = m_icon_repository.find (icon);
 
     // The icon has been loaded, just return the local copy filename.
     if (it != m_icon_repository.end ())
@@ -269,7 +279,7 @@ SocketIMEngineGlobal::load_icon (const String &icon)
     if (scim_load_file (local_icon, 0) != 0)
         return local_icon;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
     char *bufptr = 0;
     size_t filesize = 0;
@@ -296,7 +306,7 @@ SocketIMEngineGlobal::load_icon (const String &icon)
         }
 
         char tmp [80];
-        snprintf (tmp, 80, "%lu", m_socket_magic_key);
+        snprintf (tmp, 80, "%lu", (unsigned long) m_socket_magic_key);
 
         tempfile = String (SCIM_TEMPDIR) + String (SCIM_PATH_DELIM_STRING) +
                    String ("scim-") + String (tmp) + String ("-") +
@@ -334,7 +344,7 @@ SocketIMEngineGlobal::connect_reconnect_signal (Slot0<void> *slot_reconnect)
 void
 SocketIMEngineGlobal::destroy_all_icons ()
 {
-    std::map <String, String>::const_iterator it = m_icon_repository.begin ();
+    IconRepository::const_iterator it = m_icon_repository.begin ();
 
     for (; it != m_icon_repository.end (); ++ it) {
         unlink (it->second.c_str ());
@@ -346,10 +356,10 @@ SocketIMEngineGlobal::destroy_all_icons ()
 int
 SocketFactory::create_peer_instance (const String &encoding)
 {
-    SocketTransaction trans;
     int cmd;
     int si_peer_id = -1;
     uint32 val;
+    Transaction trans;
 
     SCIM_DEBUG_IMENGINE(1) << "Create IMEngine Instance " << m_peer_uuid << ".\n";
 
@@ -379,12 +389,12 @@ SocketFactory::SocketFactory (const String &peer_uuid)
       m_icon_file (String ("")),
       m_ok (false)
 {
-    SocketTransaction trans;
     String locales;
     String iconfile;
     int cmd;
     bool m_name_ok = false;
     bool m_locale_ok = false;
+    Transaction trans;
 
     SCIM_DEBUG_IMENGINE(1) << "Create SocketFactory " << peer_uuid << ".\n";
 
@@ -470,9 +480,9 @@ SocketFactory::get_name () const
 WideString
 SocketFactory::get_authors () const
 {
-    SocketTransaction trans;
     int cmd;
     WideString authors;
+    Transaction trans;
 
     SCIM_DEBUG_IMENGINE(1) << "Get Authors " << m_peer_uuid << ".\n";
 
@@ -500,9 +510,9 @@ SocketFactory::get_authors () const
 WideString
 SocketFactory::get_credits () const
 {
-    SocketTransaction trans;
     int cmd;
     WideString credits;
+    Transaction trans;
 
     SCIM_DEBUG_IMENGINE(1) << "Get Credits " << m_peer_uuid << ".\n";
 
@@ -530,9 +540,9 @@ SocketFactory::get_credits () const
 WideString
 SocketFactory::get_help () const
 {
-    SocketTransaction trans;
     int cmd;
     WideString help;
+    Transaction trans;
 
     SCIM_DEBUG_IMENGINE(1) << "Get Help " << m_peer_uuid << ".\n";
 
@@ -601,26 +611,27 @@ SocketInstance::SocketInstance (SocketFactory *factory,
 
 SocketInstance::~SocketInstance ()
 {
+    Transaction trans;
+
     SCIM_DEBUG_IMENGINE(1) << "Destroy IMEngine Instance " << m_peer_id << ".\n";
 
     m_signal_reconnect_connection.disconnect ();
 
     if (m_peer_id >= 0) {
-        SocketTransaction trans;
         global->init_transaction (trans);
 
         trans.put_command (SCIM_TRANS_CMD_DELETE_INSTANCE);
         trans.put_data (m_peer_id);
 
-        global->send_transaction (trans);
-        global->receive_transaction (trans);
+        commit_transaction (trans);
     }
 }
 
 bool
 SocketInstance::process_key_event (const KeyEvent& key)
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "process_key_event (" << m_peer_id << ")\n";
@@ -635,7 +646,8 @@ SocketInstance::process_key_event (const KeyEvent& key)
 void
 SocketInstance::move_preedit_caret (unsigned int pos)
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "move_preedit_caret (" << m_peer_id << ")\n";
@@ -650,7 +662,8 @@ SocketInstance::move_preedit_caret (unsigned int pos)
 void
 SocketInstance::select_candidate (unsigned int item)
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "select_candidate (" << m_peer_id << ")\n";
@@ -665,7 +678,8 @@ SocketInstance::select_candidate (unsigned int item)
 void
 SocketInstance::update_lookup_table_page_size (unsigned int page_size)
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "update_lookup_table_page_size (" << m_peer_id << ")\n";
@@ -680,7 +694,8 @@ SocketInstance::update_lookup_table_page_size (unsigned int page_size)
 void
 SocketInstance::lookup_table_page_up ()
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "lookup_table_page_up (" << m_peer_id << ")\n";
@@ -694,7 +709,8 @@ SocketInstance::lookup_table_page_up ()
 void
 SocketInstance::lookup_table_page_down ()
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "lookup_table_page_up (" << m_peer_id << ")\n";
@@ -708,7 +724,8 @@ SocketInstance::lookup_table_page_down ()
 void
 SocketInstance::reset ()
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "reset (" << m_peer_id << ")\n";
@@ -722,7 +739,8 @@ SocketInstance::reset ()
 void
 SocketInstance::focus_in ()
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "focus_in (" << m_peer_id << ")\n";
@@ -736,7 +754,8 @@ SocketInstance::focus_in ()
 void
 SocketInstance::focus_out ()
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "focus_out (" << m_peer_id << ")\n";
@@ -750,7 +769,8 @@ SocketInstance::focus_out ()
 void
 SocketInstance::trigger_property (const String &property)
 {
-    SocketTransaction trans;
+    Transaction trans;
+
     global->init_transaction (trans);
 
     SCIM_DEBUG_IMENGINE(1) << "trigger_property (" << m_peer_id << ", " << property << ")\n";
@@ -762,30 +782,68 @@ SocketInstance::trigger_property (const String &property)
     commit_transaction (trans);
 }
 
-bool
-SocketInstance::commit_transaction (SocketTransaction &trans)
+void
+SocketInstance::process_helper_event (const String &helper_uuid, const Transaction &helper_trans)
 {
-    SocketTransaction recv_trans;
+    Transaction trans;
 
+    global->init_transaction (trans);
+
+    SCIM_DEBUG_IMENGINE(1) << "process_helper_event (" << m_peer_id << ", " << helper_uuid << ")\n";
+
+    trans.put_command (SCIM_TRANS_CMD_PROCESS_HELPER_EVENT);
+    trans.put_data (m_peer_id);
+    trans.put_data (helper_uuid);
+    trans.put_data (helper_trans);
+
+    commit_transaction (trans);
+}
+
+void
+SocketInstance::update_client_capabilities (unsigned int cap)
+{
+    Transaction trans;
+
+    global->init_transaction (trans);
+
+    SCIM_DEBUG_IMENGINE(1) << "update_client_capabilities (" << m_peer_id << ", " << cap << ")\n";
+
+    trans.put_command (SCIM_TRANS_CMD_UPDATE_CLIENT_CAPABILITIES);
+    trans.put_data (m_peer_id);
+    trans.put_data ((uint32) cap);
+
+    commit_transaction (trans);
+}
+
+bool
+SocketInstance::commit_transaction (Transaction &trans)
+{
     SCIM_DEBUG_IMENGINE(2) << " commit_transaction:\n";
 
+    bool ret = false;
+
     if (m_peer_id >= 0) {
-        if (global->send_transaction (trans) &&
-            global->receive_transaction (recv_trans)) {
-            return do_transaction (recv_trans);
+        if (global->send_transaction (trans)) {
+            while (1) {
+                if (!global->receive_transaction (trans)) break;
+                if (!do_transaction (trans, ret)) return ret;
+            }
         }
     }
 
     if (global->create_connection ())
         reset ();
 
-    return false;
+    return ret;
 }
 
 bool
-SocketInstance::do_transaction (SocketTransaction &trans)
+SocketInstance::do_transaction (Transaction &trans, bool &ret)
 {
     int cmd;
+    bool cont = false;
+
+    ret = false;
 
     SCIM_DEBUG_IMENGINE(2) << " Do transaction:\n";
 
@@ -911,20 +969,92 @@ SocketInstance::do_transaction (SocketTransaction &trans)
                     }
                     break;
                 }
+                case SCIM_TRANS_CMD_BEEP:
+                {
+                    SCIM_DEBUG_IMENGINE(3) << "  beep ()\n";
+                    beep ();
+                    break;
+                }
+                case SCIM_TRANS_CMD_START_HELPER:
+                {
+                    String helper_uuid;
+                    if (trans.get_data (helper_uuid)) {
+                        SCIM_DEBUG_IMENGINE(3) << "  start_helper (" << helper_uuid << ")\n";
+                        start_helper (helper_uuid);
+                    }
+                    break; 
+                }
+                case SCIM_TRANS_CMD_STOP_HELPER:
+                {
+                    String helper_uuid;
+                    if (trans.get_data (helper_uuid)) {
+                        SCIM_DEBUG_IMENGINE(3) << "  stop_helper (" << helper_uuid << ")\n";
+                        stop_helper (helper_uuid);
+                    }
+                    break; 
+                }
+                case SCIM_TRANS_CMD_SEND_HELPER_EVENT:
+                {
+                    String helper_uuid;
+                    Transaction temp_trans;
+                    if (trans.get_data (helper_uuid) && trans.get_data (temp_trans)) {
+                        SCIM_DEBUG_IMENGINE(3) << "  send_helper_event (" << helper_uuid << ")\n";
+                        send_helper_event (helper_uuid, temp_trans);
+                    }
+                    break; 
+                }
                 case SCIM_TRANS_CMD_OK:
                 {
                     SCIM_DEBUG_IMENGINE(3) << "  ret = true\n";
-                    return true;
+                    ret = true;
+                    break;
                 }
-                default:
+                case SCIM_TRANS_CMD_GET_SURROUNDING_TEXT:
                 {
-                    SCIM_DEBUG_IMENGINE(3) << "  ret = false\n";
-                    return false;
+                    WideString text;
+                    int cursor;
+                    uint32 maxlen_before;
+                    uint32 maxlen_after;
+                    Transaction temp_trans;
+                    if (trans.get_data (maxlen_before) && trans.get_data (maxlen_after)) {
+                        global->init_transaction (temp_trans);
+                        if (get_surrounding_text (text, cursor, (int) maxlen_before, (int) maxlen_after)) {
+                            temp_trans.put_command (SCIM_TRANS_CMD_GET_SURROUNDING_TEXT);
+                            temp_trans.put_data (text);
+                            temp_trans.put_data ((uint32) cursor);
+                        } else {
+                            temp_trans.put_command (SCIM_TRANS_CMD_FAIL);
+                        }
+                        global->send_transaction (temp_trans);
+                    }
+                    cont = true;
+                    break;
+                }
+                case SCIM_TRANS_CMD_DELETE_SURROUNDING_TEXT:
+                {
+                    uint32 offset;
+                    uint32 len;
+                    Transaction temp_trans;
+                    if (trans.get_data (offset) && trans.get_data (len)) {
+                        global->init_transaction (temp_trans);
+                        if (delete_surrounding_text ((int) offset, (int) len)) {
+                            temp_trans.put_command (SCIM_TRANS_CMD_DELETE_SURROUNDING_TEXT);
+                            temp_trans.put_command (SCIM_TRANS_CMD_OK);
+                        } else {
+                            temp_trans.put_command (SCIM_TRANS_CMD_FAIL);
+                        }
+                        global->send_transaction (temp_trans);
+                    }
+                    cont = true;
+                    break;
                 }
             }
         }
     }
-    return false;
+
+    SCIM_DEBUG_IMENGINE(2) << " End of Do transaction\n";
+
+    return cont;
 }
 
 void
@@ -933,6 +1063,7 @@ SocketInstance::reconnect_callback (void)
     m_peer_id = m_factory->create_peer_instance (get_encoding ());
 }
 
+} // namespace scim
 
 /*
 vi:ts=4:nowrap:ai:expandtab
