@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_imengine_setup.cpp,v 1.8 2005/05/28 13:54:57 suzhe Exp $
+ * $Id: scim_imengine_setup.cpp,v 1.9 2005/08/15 06:33:26 suzhe Exp $
  *
  */
 
@@ -624,13 +624,6 @@ get_factory_list (const ConfigPointer &config,
     langs.clear ();
     icons.clear ();
 
-    // Only load socket IMEngine module if socket Config module is used.
-    if (config->get_name () == String ("socket") &&
-        std::find (module_list.begin (), module_list.end (), String ("socket")) != module_list.end ()) {
-        module_list.clear ();
-        module_list.push_back (String ("socket"));
-    }
-
     // Add "English/European" factory first.
     factory = new ComposeKeyFactory ();
     uuids.push_back (factory->get_uuid ());
