@@ -1,8 +1,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2003 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_config_base.cpp,v 1.16 2004/07/02 14:06:37 suzhe Exp $
+ * $Id: scim_config_base.cpp,v 1.19 2005/04/09 15:38:39 suzhe Exp $
  */
 
 #define Uses_SCIM_CONFIG_BASE
@@ -50,15 +49,14 @@ _create_config (const String &default_module)
     }
 
     if (_scim_default_config_module.valid ()) {
-        _scim_default_config = _scim_default_config_module.create_config (String ("scim"));
+        _scim_default_config = _scim_default_config_module.create_config ();
         return _scim_default_config;
     }
 
     return ConfigPointer (0);
 }
 
-ConfigBase::ConfigBase (const String& app_name)
-    : m_app_name (app_name)
+ConfigBase::ConfigBase ()
 {
 }
 
@@ -69,7 +67,7 @@ ConfigBase::~ConfigBase ()
 bool
 ConfigBase::valid () const
 {
-    return !m_app_name.empty();
+    return true;
 }
 
 String
@@ -144,19 +142,6 @@ ConfigBase::read (const String& key, const std::vector <int>& defVal) const
     return tmp;
 }
 
-const String&
-ConfigBase::get_app_name() const
-{
-    return m_app_name;
-}
-
-void
-ConfigBase::set_app_name(const String& name)
-{
-    if (name.length ())
-        m_app_name = name;
-}
-
 bool
 ConfigBase::reload ()
 {
@@ -195,8 +180,7 @@ ConfigBase::get (bool create_on_demand, const String &default_module)
  * Implementation of DummyConfig
  */
 
-DummyConfig::DummyConfig (const String& app_name)
-    : ConfigBase (app_name)
+DummyConfig::DummyConfig ()
 {
 }
 
@@ -210,6 +194,12 @@ DummyConfig::valid () const
     return ConfigBase::valid();
 }
 
+String
+DummyConfig::get_name () const
+{
+    return "dummy";
+}
+
 // String
 bool
 DummyConfig::read (const String& key, String *pStr) const
