@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_iconv.cpp,v 1.17 2005/01/10 08:30:54 suzhe Exp $
+ * $Id: scim_iconv.cpp,v 1.18 2005/08/15 12:45:46 suzhe Exp $
  *
  */
 
@@ -126,6 +126,12 @@ IConvert::set_encoding (const String& encoding)
     return true;
 }
 
+String
+IConvert::get_encoding () const
+{
+    return m_impl->m_encoding;
+}
+
 bool
 IConvert::convert (String &dest, const ucs4_t *src, int src_len) const
 {
