@@ -23,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket.cpp,v 1.43 2005/01/25 15:13:15 suzhe Exp $
+ * $Id: scim_socket.cpp,v 1.44 2005/12/01 08:27:36 suzhe Exp $
  *
  */
 
@@ -1144,20 +1144,46 @@ String scim_get_default_panel_socket_address (const String &display)
     if (address == "default")
         address = SCIM_DEFAULT_PANEL_SOCKET_ADDRESS;
 
-    String::size_type colon_pos = display.rfind (':');
+    SocketAddress sockaddr (address);
 
+    if (!sockaddr.valid ())
+        return String ();
+
+    String::size_type colon_pos = display.rfind (':');
     String disp_name = display;
+    int    disp_num  = 0;
 
     // Maybe It's a X11 DISPLAY name
     if (colon_pos != String::npos) {
-        String::size_type dot_pos = display.find ('.', colon_pos);
+        String::size_type dot_pos = display.find ('.', colon_pos+1);
         // It has screen number
         if (dot_pos != String::npos) {
             disp_name = display.substr (0, dot_pos);
         }
+        // FIXME: ignore remote X Server name.
+        disp_num = atoi (display.substr (colon_pos + 1, dot_pos - colon_pos - 1).c_str ());
     }
 
-    return address + String ("-") + disp_name;
+    if (sockaddr.get_family () == SCIM_SOCKET_LOCAL) {
+        address = address + disp_name;
+    } else if (sockaddr.get_family () == SCIM_SOCKET_INET) {
+        std::vector <String> varlist;
+        scim_split_string_list (varlist, address, ':');
+        if (varlist.size () == 3) {
+            int port = atoi (varlist [2].c_str ()) + disp_num;
+            char buf [10];
+            snprintf (buf, 10, "%d", port);
+            varlist [2] = String (buf);
+            address = scim_combine_string_list (varlist, ':');
+        }
+    }
+
+    sockaddr.set_address (address);
+
+    if (sockaddr.valid ())
+        return address;
+
+    return String ();
 }
 
 String scim_get_default_helper_manager_socket_address ()
