@@ -1,7 +1,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_utility.cpp,v 1.37.2.1 2004/12/01 14:48:13 suzhe Exp $
+ * $Id: scim_utility.cpp,v 1.48 2005/04/30 01:45:03 liuspider Exp $
  */
 
 #define Uses_SCIM_UTILITY
@@ -433,7 +433,7 @@ scim_if_wchar_ucs4_equal ()
     return false;
 }
 
-struct {
+static struct {
     ucs4_t half;
     ucs4_t full;
     ucs4_t size;
@@ -583,10 +583,26 @@ scim_get_user_name ()
     return String (uid_str);
 }
 
+String
+scim_get_user_data_dir ()
+{
+    String dir = scim_get_home_dir () + String ("/.scim");
+    scim_make_dir (dir);
+    return dir;
+}
+
 String
 scim_get_current_locale ()
 {
-    return String (setlocale (LC_CTYPE, 0));
+    char *locale = setlocale (LC_CTYPE, 0);
+
+    if (locale) return String (locale);
+    return String ();
+}
+
+String scim_get_current_language ()
+{
+    return scim_get_locale_language (scim_get_current_locale ());
 }
 
 bool
@@ -740,7 +756,7 @@ static __Language __languages [] = {
     { "pa_IN",    NULL, N_("Panjabi"), NULL, NULL },
     { "pl_PL",    NULL, N_("Polish"), "Polski", NULL },
     { "pt",    "pt_PT", N_("Portuguese"), "Português", NULL },
-    { "pt_BR",    NULL, N_("Portuguese (Brazilian)"), "Português do Brasil", NULL },
+    { "pt_BR",    NULL, N_("Portuguese (Brazil)"), "Português do Brasil", NULL },
     { "pt_PT",    NULL, N_("Portuguese"), "Português", "@euro" },
     { "ro_RO",    NULL, N_("Romanian"), "Română", NULL },
     { "ru_RU",    NULL, N_("Russian"), "русский", ".koi8r" },
@@ -877,7 +893,10 @@ scim_get_language_locales (const String &lang)
 String
 scim_get_locale_language (const String &locale)
 {
-    return scim_validate_language (locale.substr (0, locale.find ('.')));
+    if (locale.length () == 0) return String ();
+
+    String str = locale.substr (0, locale.find ('.'));
+    return scim_validate_language (str.substr (0, str.find ('@')));
 }
 
 String
@@ -910,39 +929,33 @@ scim_get_normalized_language (const String &lang)
  #define SCIM_LAUNCHER  (SCIM_LIBEXECDIR "/scim-launcher")
 #endif
 
-#define SCIM_LAUNCHER_DAEMON (SCIM_LAUNCHER "-daemon")
-
 int  scim_launch (bool          daemon,
                   const String &config,
                   const String &imengines,
                   const String &frontend,
-                  int           verbose,
-                  const String &debug_mask,
-                  const String &debug_output,
-                  char  *const argv [])
+                  char  * const argv [])
 {
-    if (!config.length () || !imengines.length () || !frontend.length () || !debug_mask.length () || !debug_output.length ())
+    if (!config.length () || !imengines.length () || !frontend.length ())
         return -1;
 
-    int   new_argc = 7;
-    char * new_argv [80];
+    int   new_argc = 0;
+    char *new_argv [40];
     char verbose_buf [10];
 
-    if (daemon) new_argv [0] = strdup (SCIM_LAUNCHER_DAEMON);
-    else new_argv [0] = strdup (SCIM_LAUNCHER);
+    new_argv [new_argc ++] = strdup (SCIM_LAUNCHER);
 
-    new_argv [1] = strdup (config.c_str ());
-    new_argv [2] = strdup (imengines.c_str ());
-    new_argv [3] = strdup (frontend.c_str ());
+    if (daemon)
+        new_argv [new_argc ++] = strdup ("-d");
 
-    snprintf (verbose_buf, 10, "%d", verbose);
-
-    new_argv [4] = strdup (verbose_buf);
-    new_argv [5] = strdup (debug_mask.c_str ());
-    new_argv [6] = strdup (debug_output.c_str ());
+    new_argv [new_argc ++] = strdup ("-c");
+    new_argv [new_argc ++] = strdup (config.c_str ());
+    new_argv [new_argc ++] = strdup ("-e");
+    new_argv [new_argc ++] = strdup (imengines.c_str ());
+    new_argv [new_argc ++] = strdup ("-f");
+    new_argv [new_argc ++] = strdup (frontend.c_str ());
 
     if (argv) {
-        for (int i = 0; argv [i] && i < 76; ++i, ++new_argc)
+        for (int i = 0; argv [i] && new_argc < 40 ; ++i, ++new_argc)
             new_argv [new_argc] = strdup (argv [i]);
     }
 
@@ -982,8 +995,7 @@ int  scim_launch (bool          daemon,
 
 int scim_launch_panel (bool          daemon,
                        const String &config,
-                       int           verbose,
-                       const String &debug_output,
+                       const String &display,
                        char * const  argv [])
 {
     if (!config.length ())
@@ -1000,27 +1012,26 @@ int scim_launch_panel (bool          daemon,
                         panel_program;
     }
 
-    int   new_argc = 5;
+    //if the file is not exist or is not executable, fallback to default
+    if (access (panel_program.c_str (), X_OK) != 0)
+            panel_program = String (SCIM_PANEL_PROGRAM);
+
+    int   new_argc = 0;
     char *new_argv [80];
 
-    new_argv [0] = strdup (panel_program.c_str ());
-    new_argv [1] = strdup ("-c");
-    new_argv [2] = strdup (config.c_str ());
-    new_argv [3] = strdup ("-o");
+    new_argv [new_argc ++] = strdup (panel_program.c_str ());
 
-    if (debug_output.length ()) 
-        new_argv [4] = strdup (debug_output.c_str ());
-    else
-        new_argv [4] = strdup ("none");
+    new_argv [new_argc ++] = strdup ("--display");
+    new_argv [new_argc ++] = strdup (display.c_str ());
+
+    new_argv [new_argc ++] = strdup ("-c");
+    new_argv [new_argc ++] = strdup (config.c_str ());
 
     if (daemon)
         new_argv [new_argc ++] = strdup ("-d");
 
-    for (int i = 0; i < verbose && i < 10; ++ i) 
-        new_argv [new_argc ++] = strdup ("-v");
-
     if (argv) {
-        for (int i = 0; argv [i] && i < 50; ++i, ++new_argc)
+        for (int i = 0; argv [i] && new_argc < 40; ++i, ++new_argc)
             new_argv [new_argc] = strdup (argv [i]);
     }
 
@@ -1082,6 +1093,36 @@ scim_usleep (unsigned int usec)
 #endif
 }
 
+void scim_daemon ()
+{
+#if HAVE_DAEMON
+    if (daemon (0, 0) == -1)
+        std::cerr << "Error to make SCIM into a daemon!\n";
+
+    return;
+#else        
+    pid_t id;
+ 
+    id = fork ();
+    if (id == -1) {
+        std::cerr << "Error to make SCIM into a daemon!\n";
+        return;
+    } else if (id > 0) {
+        exit (0);
+    }
+
+    id = fork ();
+    if (id == -1) {
+        std::cerr << "Error to make SCIM into a daemon!\n";
+        return;
+    } else if (id > 0) {
+        exit (0);
+    }
+
+    return;
+#endif
+}
+
 } // namespace scim
 
 /*
