@@ -5,9 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2004 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2003 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -25,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_utility.h,v 1.28.2.1 2004/09/28 05:18:32 liuspider Exp $
+ * $Id: scim_utility.h,v 1.36 2005/04/09 15:38:39 suzhe Exp $
  */
 
 #ifndef __SCIM_UTILITY_H
@@ -33,7 +31,7 @@
 
 namespace scim {
 /**
- * @addtogroup Helper
+ * @addtogroup Accessories
  * @{
  */
 
@@ -232,6 +230,12 @@ String scim_get_locale_encoding (const String& locale);
  */
 String scim_get_current_locale ();
 
+/**
+ * @brief Get current system language.
+ * @return The current system language.
+ */
+String scim_get_current_language ();
+
 /**
  * @brief Get the max length of the multibyte char of a locale.
  *
@@ -290,6 +294,11 @@ String scim_get_home_dir ();
  */
 String scim_get_user_name ();
 
+/**
+ * @brief Get SCIM data dir of current user.
+ */
+String scim_get_user_data_dir ();
+
 /**
  * @brief Load a file into memory.
  *
@@ -381,9 +390,6 @@ String scim_get_normalized_language (const String &lang);
  * @param config        The Config module to be used.
  * @param imengines     The IMEngines to be loaded, separated by comma.
  * @param frontend      The FrontEnd module to be used.
- * @param verbose       The debug verbose level.
- * @param debug_mask    The debug mask.
- * @param debug_output  The debug output target, can be stdout, stderr or a file.
  * @param argv          Additional arguments passed to the new process's FrontEnd. Must
  *                      terminated by a NULL pointer.
  *
@@ -394,9 +400,6 @@ int  scim_launch (bool          daemon,
                   const String &config,
                   const String &imengines,
                   const String &frontend,
-                  int           verbose = 0,
-                  const String &debug_mask = "none",
-                  const String &debug_output = "none",
                   char  * const argv [] = 0);
 
 /**
@@ -406,8 +409,8 @@ int  scim_launch (bool          daemon,
  *                      otherwise the current process will be stopped until 
  *                      the newly created process exit.
  * @param config        The Config module to be used.
- * @param verbose       The debug verbose level.
- * @param debug_output  The debug output target, can be stdout, stderr or a file.
+ * @param display       The display name on which the panel runs.
+ *                      eg. for X11 : localhost:0.0
  * @param argv          Additional arguments passed to the new process's FrontEnd. Must
  *                      terminated by a NULL pointer.
  *
@@ -416,8 +419,7 @@ int  scim_launch (bool          daemon,
  */
 int scim_launch_panel (bool          daemon,
                        const String &config,
-                       int           verbose = 0,
-                       const String &debug_output = "none",
+                       const String &display,
                        char * const  argv [] = 0);
 
 /**
@@ -427,6 +429,11 @@ int scim_launch_panel (bool          daemon,
  */
 void scim_usleep (unsigned int usec);
 
+/**
+ * @brief Switch process into daemon mode.
+ */
+void scim_daemon ();
+        
 /** @} */
 } // namespace scim
 
